/* Copyright Erno de Weerd
 * 2002
 * AirKnow@hotmail.com
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;
using AirKnow.AL.Creatures;

namespace AirKnow.AL.Controls
{
	public class CreaturePlayField : System.Windows.Forms.PictureBox
	{
		private int minX, maxX, minY, maxY;
		private Bitmap field;
		private Color fieldColor;
		private InterpolationMode style;
		private ArrayList creatures = null;
		private bool initialized = false;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Color FieldColor
		{
			get
			{
				return fieldColor;
			}
			set
			{
				for(int y=0; y<field.Height; y++)
					for(int x=0; x<field.Width; x++)
						if(field.GetPixel(x, y).ToArgb() == fieldColor.ToArgb())
							field.SetPixel(x, y, value);
					
				fieldColor = value;
				this.Invalidate();
			}
		}
		
		public InterpolationMode Style
		{
			get
			{
				return style;
			}
			set
			{
				style = value;
				this.Invalidate();
			}
		}

		public ArrayList Creatures
		{
			get
			{
				return creatures;
			}
			set
			{
				creatures = value;
			}
		}


		public CreaturePlayField()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitForm call
			field = new Bitmap(this.Width<<1, this.Height<<1);
			Clear();
		}

		public void Clear()
		{
			for(int y=0;y<field.Height;y++)
				for(int x=0;x<field.Width;x++)
					field.SetPixel(x,y,FieldColor);

			int top = 0, left = 0, bottom = 0, right = 0;
			if(creatures != null && creatures.Count > 0)
			{
				for(int index = 0; index < Creatures.Count; index++)
				{
					Creature creature = Creatures[index] as Creature;
					if(creature != null)
					{
						if(index == 0)
						{
							top = bottom = creature.Y;
							left = right = creature.X;
						}
						else
						{
							if(creature.X < left)
								left = creature.X;
							if(creature.X > right)
								right = creature.X;
							if(creature.Y < top)
								top = creature.Y;
							if(creature.Y > bottom)
								bottom = creature.Y;
						}
					}
				}
				minX = left;
				maxX = right;
				minY = top;
				maxY = bottom;
			}
			else
			{
				minX = this.Width - 1;
				maxX = this.Width + 1;
				minY = this.Height - 1;
				maxY = this.Height + 1;
			}
		}


		public void AdjustViewTo(int x, int y)
		{
			if(x <= minX && x > 1)
				minX = x - 1;

			if(x > maxX && x < field.Width - 1)
				maxX = x + 1;

			if(y <= minY && y > 0)
				minY = y - 1;

			if(y > maxY && y < field.Height - 1)
				maxY = y + 1;
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// PlayField
			// 
			this.Name = "PlayField";

		}
		#endregion

		protected override void OnPaint(PaintEventArgs e)
		{
			RectangleF src = new RectangleF(minX, minY ,maxX - minX + 1, maxY - minY + 1);
			Bitmap b = field.Clone(src, PixelFormat.DontCare);
			
			if(creatures != null)
			{
				for(int index = 0; index < Creatures.Count; index++)
				{
					Creature creature = Creatures[index] as Creature;
					if(creature != null)
					{
						if(creature.X - minX >= 0 && creature.X - minX <= maxX &&
							creature.Y -minY >= 0 && creature.Y - minY <= maxY)
						{
							b.SetPixel(creature.X-minX, creature.Y-minY, creature.Color);
						}
					}
				}
			}

			RectangleF dest = new RectangleF(0, 0, this.Width, this.Height);
			src = new RectangleF(0, 0, b.Width, b.Height);
			e.Graphics.InterpolationMode = Style;
			e.Graphics.DrawImage(b, dest, src, GraphicsUnit.Pixel);
		}
		
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);
			
			if(!initialized)
			{
				initialized = true;
				Bitmap b = new Bitmap(field, this.Width<<1, this.Height<<1);
				minX = this.Width - 1;
				maxX = this.Width + 1;
				minY = this.Height - 1;
				maxY = this.Height + 1;
				field = b;
			}
			
			//this.Invalidate();
		}

		public bool PersistentPlot(int x, int y, Color color)
		{
			if(x >= 0 && x < field.Width && y >= 0 && y < field.Height)
			{
				field.SetPixel(x, y, color);
				AdjustViewTo(x, y);
				return true;
			}
			else
			{
				return false;
			}
		}

		public Color Read(int x, int y)
		{
			if(x >= 0 && x < field.Width && y >= 0 && y < field.Height)
			{
				return field.GetPixel(x, y);
			}
			else
			{
				throw new ArgumentOutOfRangeException();
			}
		}

	}
}
