VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAntGrid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' ---------------------------------------------------------
'
' Class AntGrid (clsAntGrid.cls)
'
' Purpose:
'   Represents the grid in which Langton's Ant(s) moves through.
'
' ---------------------------------------------------------


'
' Public Types
'


' Represents a square in the grid.
Public Enum GridSquare
    White
    Black
End Enum

' Determines the current demo mode.
Public Enum DemoMode
    Stopped
    Running
    Paused
    FastForward
End Enum


'
' Private API Declarations
'


Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Private Declare Function FillRect Lib "user32" (ByVal hdc As Long, lpRect As RECT, ByVal hBrush As Long) As Long

Private Declare Function MoveToEx Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, lpPoint As POINTAPI) As Long
Private Declare Function LineTo Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long) As Long

Private Declare Function GetTickCount Lib "kernel32" () As Long


'
' Private Types
'


' API related user-defined types.

Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type POINTAPI
    X As Long
    Y As Long
End Type


'
' Private Variables
'


' The two-dimensional array which contains the ant grid itself.
Private mgsqGrid() As GridSquare
' The dimensions of the grid.
Private mlngGridWidth As Long
Private mlngGridHeight As Long
' The collection of ants which reside inside the grid.
Private mcolAnts As Collection
' The total number of rounds the grid has been through since it was first ran.
Private mlngTotalRounds As Long
' Used for storing the height & width of the grid square to be drawn.
Private mlngSqrWidth As Long, mlngSqrHeight As Long
' The current mode of the Langton's Ant demo (stopped, running, paused, fastForward).
Private mmodCurrentDemoMode As DemoMode


' Grid controls.

' The picture box on which the grid is drawn.
Private WithEvents mpicCanvas As PictureBox
Attribute mpicCanvas.VB_VarHelpID = -1
' Vertical and horizontal scroll bars used for scrolling the canvas.
Private WithEvents mvsbVerticalSB As VScrollBar
Attribute mvsbVerticalSB.VB_VarHelpID = -1
Private WithEvents mhsbHorizontalSB As HScrollBar
Attribute mhsbHorizontalSB.VB_VarHelpID = -1
' The controls collection of a form passed using the Initialize procedure.
Private mvntControls As Variant
' The width\height of the verical\horizontal scroll bars.
Private Const VSB_WIDTH As Integer = 255
Private Const HSB_HEIGHT As Integer = 255
' The message to be displayed instead of the grid when the demo is in fastForward mode.
Private Const FASTForward_MSG As String = "The Langton's Ant Demo is in FastForward Mode"


' Various grid settings.

' The maxmium no. of grid squares which fit horizontallty\vertically into the canvas.
Private mintGridScale As Integer
' Determines whether the grid lines should be drawn.
Private mblnShowGridLines As Boolean
' The delay (in milliseconds) between each round.
Private mlngRoundDelay As Long
' Pointers to brushes which indicate the colors of the various parts of the grid.
Private mlngBlackSqrBrush As Long, mlngWhiteSqrBrush As Long, mlngAntBrush As Long
' The colors of the various parts of the grid.
Private mlngBlackSqrColor As Long, mlngWhiteSqrColor As Long, _
    mlngAntColor As Long, mlngGridLinesColor As Long
' Determines whether the square where the ant is currently at should be painted in a unique color.
Private mblnShowAntPos As Boolean
' Used to indicate whether the grid been initialized yet.
Private mblnInitialized As Boolean
' Used for allowing the user to disable\enable the grid.
Private mblnEnabled As Boolean


'
' Public Properties
'


' Ant Property (Read Only)
Public Property Get Ant(Index As Variant) As clsAnt
    If (IsNumeric(Index)) Then
        ' The Index variable represents the ant's index number in the collection.
        
        If ((Index > 0) And (Index <= mcolAnts.Count)) Then
            Set Ant = mcolAnts.Item(Index)
        Else
            Set Ant = Nothing
        End If
    Else
        ' The Index variable represents the ant's key in the collection.
        If (Index <> "") Then
            Set Ant = mcolAnts.Item(Index)
        Else
            Set Ant = Nothing
        End If
    End If
End Property

' AntCount Property (Read Only)
Public Property Get AntCount() As Integer
    AntCount = mcolAnts.Count
End Property

' DemoMode Property
Public Property Get DemoMode() As DemoMode
    DemoMode = mmodCurrentDemoMode
End Property

Public Property Let DemoMode(NewMode As DemoMode)
    mmodCurrentDemoMode = NewMode
    
    If (NewMode = Stopped) Then
        ' Reset the entire ant grid.
        ReDim mgsqGrid(mlngGridWidth, mlngGridHeight)
        
        ' Redraw the grid.
        DrawGrid
        
    ElseIf (NewMode = Running) Then
        ' Redraw the grid.
        DrawGrid
        
        ' Run the demo.
        RunDemo
        
    ElseIf (mmodCurrentDemoMode = FastForward) Then
        ' Display a message saying the demo is in a fastForward mode.
        mpicCanvas.Cls
        mpicCanvas.ForeColor = vbBlack
        mpicCanvas.CurrentX = (mpicCanvas.Width / 2) - (mpicCanvas.TextWidth(FASTForward_MSG) / 2)
        mpicCanvas.CurrentY = (mpicCanvas.Height / 2) - (mpicCanvas.TextHeight(FASTForward_MSG) / 2)
        mpicCanvas.Print FASTForward_MSG
    
        ' Fast Forward the demo.
        FastForwardDemo
    Else
        ' Redraw the grid.
        DrawGrid
    End If
End Property

' Enabled Property
Public Property Get Enabled() As Boolean
    Enabled = mblnEnabled
End Property

Public Property Let Enabled(NewValue As Boolean)
    mblnEnabled = NewValue
    
    ' Redraw the grid if the grid was just enabled.
    If (NewValue = True) Then DrawGrid
End Property

' Grid Property
Public Property Get Grid(X As Long, Y As Long) As GridSquare
    If ((X >= 0) And (X < mlngGridWidth) And (Y >= 0) And (Y < mlngGridHeight)) Then
        Grid = mgsqGrid(X, Y)
    Else
        Grid = White
    End If
End Property

Public Property Let Grid(X As Long, Y As Long, NewSquare As GridSquare)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    If ((X >= 0) And (X < mlngGridWidth) And (Y >= 0) And (Y < mlngGridHeight) And _
        ((NewSquare = Black) Or (NewSquare = White))) Then
        mgsqGrid(X, Y) = NewSquare
    End If
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode

End Property

' GridWidth Property (Read Only)
Public Property Get GridWidth() As Long
    GridWidth = mlngGridWidth
End Property

' GridHeight Property (Read Only)
Public Property Get GridHeight() As Long
    GridHeight = mlngGridHeight
End Property

' GridScale Property
Public Property Get GridScale() As Integer
    GridScale = mintGridScale
End Property

Public Property Let GridScale(NewScale As Integer)
Dim modCurrMode As DemoMode

    ' See if the new scale is within the appropriate boundaries.
    If ((NewScale < 100) Or (((NewScale - 100) / (100 / mlngGridWidth)) > mlngGridWidth) Or _
        (((NewScale - 100) / (100 / mlngGridHeight)) > mlngGridHeight)) Then _
        Exit Property

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    ' Set the new scale.
    mintGridScale = NewScale
    
    ' Set the new dimensions of the grid square.
    mlngSqrWidth = mpicCanvas.ScaleX(mpicCanvas.Width, vbTwips, vbPixels) / _
        IIf((mlngGridWidth - ((NewScale - 100) / (100 / mlngGridWidth))) > 0, (mlngGridWidth - ((NewScale - 100) / (100 / mlngGridWidth))), 1)
    mlngSqrHeight = mpicCanvas.ScaleY(mpicCanvas.Height, vbTwips, vbPixels) / _
        IIf((mlngGridHeight - ((NewScale - 100) / (100 / mlngGridHeight))) > 0, (mlngGridHeight - ((NewScale - 100) / (100 / mlngGridHeight))), 1)
    
    ' Resize the horizontal and vertical scroll bars.
     
    ' Set the flag in order to stop the mhsbHorizontalSB_Change and
    ' mvsbVerticalSB_Change subs to call the DrawGrid procedure.
    mblnInitialized = False
     
     If ((NewScale > 100) And CInt((mintGridScale - 100) / (100 / mlngGridWidth)) > 0) Then
        mhsbHorizontalSB.Enabled = True
        mhsbHorizontalSB.Max = ((mintGridScale - 100) / (100 / mlngGridWidth)) + 1
        mhsbHorizontalSB.Min = 1
    
        mhsbHorizontalSB.SmallChange = 1
        mhsbHorizontalSB.LargeChange = 10
    Else
        mhsbHorizontalSB.Enabled = False
        mhsbHorizontalSB.Min = 1
        mhsbHorizontalSB.Max = 1
    End If
    
     If ((NewScale > 100) And CInt((mintGridScale - 100) / (100 / mlngGridHeight)) > 0) Then
        mvsbVerticalSB.Enabled = True
        mvsbVerticalSB.Max = ((mintGridScale - 100) / (100 / mlngGridHeight)) + 1
        mvsbVerticalSB.Min = 1
        
        mvsbVerticalSB.SmallChange = 1
        mvsbVerticalSB.LargeChange = 10
    Else
        mvsbVerticalSB.Enabled = False
        mvsbVerticalSB.Min = 1
        mvsbVerticalSB.Max = 1
    End If
    
    mblnInitialized = True
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' RoundDelay Property
Public Property Get RoundDelay() As Long
    RoundDelay = mlngRoundDelay
End Property

Public Property Let RoundDelay(NewDelay As Long)
    If (NewDelay >= 0) Then _
        mlngRoundDelay = NewDelay
End Property

' TotalRounds Property (Read Only)
Public Property Get TotalRounds() As Long
    TotalRounds = mlngTotalRounds
End Property

' ShowGridLines Property
Public Property Get ShowGridLines() As Boolean
    ShowGridLines = mblnShowGridLines
End Property

Public Property Let ShowGridLines(NewOption As Boolean)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    mblnShowGridLines = NewOption

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' ShowGridLines Property
Public Property Get ShowAntPos() As Boolean
    ShowAntPos = mblnShowAntPos
End Property

Public Property Let ShowAntPos(NewOption As Boolean)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    mblnShowAntPos = NewOption

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' AntColor Property
Public Property Get AntColor() As Long
    AntColor = mlngAntColor
End Property

Public Property Let AntColor(NewColor As Long)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    mlngAntColor = NewColor
    
    ' Create a new brush of the specified color.
    mlngAntBrush = CreateSolidBrush(NewColor)

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' BlackSqrColor Property
Public Property Get BlackSqrColor() As Long
    BlackSqrColor = mlngBlackSqrColor
End Property

Public Property Let BlackSqrColor(NewColor As Long)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    mlngBlackSqrColor = NewColor

    ' Create a new brush of the specified color.
    mlngBlackSqrBrush = CreateSolidBrush(NewColor)

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' WhiteSqrColor Property
Public Property Get WhiteSqrColor() As Long
    WhiteSqrColor = mlngWhiteSqrColor
End Property

Public Property Let WhiteSqrColor(NewColor As Long)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    mlngWhiteSqrColor = NewColor
    
    ' Create a new brush of the specified color.
    mlngWhiteSqrBrush = CreateSolidBrush(NewColor)

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property

' GridLinesColor Property
Public Property Get GridLinesColor() As Long
    GridLinesColor = mlngGridLinesColor
End Property

Public Property Let GridLinesColor(NewColor As Long)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    mlngGridLinesColor = NewColor

    ' Repaint the grid.
    DrawGrid
    
    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Property


'
' Public Subs
'


'
' Procedure Initialize
'
' Purpose:
'   Initializes the entire ant grid.
'
' Parameters:
'   Width, Height : The width & height of the grid.
'   Canvas : The canvas on which the grid will be drawn.
'   Controls : The controls collection of the form in which the container resides.
'
Public Sub Initialize(Width As Long, Height As Long, Canvas As PictureBox, Controls As Variant)
    If ((Width <= 0) Or (Height <= 0)) Then _
        Exit Sub
        
    mmodCurrentDemoMode = Stopped
    
    ' Indicate that the grid hasn't been initialized yet.
    mblnInitialized = False
    
    mlngGridWidth = Width
    mlngGridHeight = Height
    ReDim mgsqGrid(Width, Height)
    
    Set mcolAnts = New Collection
    
    mlngTotalRounds = 0
    
    ' Create default settings.
    
    mblnShowGridLines = True
    mlngRoundDelay = 100
    mintGridScale = 100 ' 100 percent.
    
    mlngBlackSqrColor = vbBlack
    mlngWhiteSqrColor = vbWhite
    mlngAntColor = vbRed
    mlngGridLinesColor = &H8000000F  ' Gray.
    
    ' Create the appropriate brushes for the colors.
    mlngBlackSqrBrush = CreateSolidBrush(mlngBlackSqrColor)
    mlngWhiteSqrBrush = CreateSolidBrush(mlngWhiteSqrColor)
    mlngAntBrush = CreateSolidBrush(mlngAntColor)

    mblnShowAntPos = True
    
    ' Enable the grid.
    mblnEnabled = True
    
    ' Create the container, canvas and scroll bars and place them inside the container.
    
    Set mvntControls = Controls
    
    Set mpicCanvas = Canvas
    Set mvsbVerticalSB = Controls.Add("VB.VScrollBar", "AntVSB")
    Set mhsbHorizontalSB = Controls.Add("VB.HScrollBar", "AntHSB")
    
    Set mvsbVerticalSB.Container = mpicCanvas
    Set mhsbHorizontalSB.Container = mpicCanvas
    
    mpicCanvas.Appearance = 0
    mpicCanvas.BorderStyle = 0
    
    mhsbHorizontalSB.Enabled = False
    mvsbVerticalSB.Enabled = False
    
    mhsbHorizontalSB.Min = 1
    mhsbHorizontalSB.Max = 1
    
    mvsbVerticalSB.Min = 1
    mvsbVerticalSB.Max = 1
    
    mblnInitialized = True
    
    ' Resize the controls.
    ResizeControls
    
    mblnInitialized = False
    
    ' Prepare the canvas for drawing.
    mpicCanvas.AutoRedraw = True
    mpicCanvas.BackColor = vbWhite
    mpicCanvas.Cls

    ' Set the initial dimensions of the grid square.
    mlngSqrWidth = mpicCanvas.ScaleX(mpicCanvas.Width, vbTwips, vbPixels) / _
        IIf((mlngGridWidth - ((mintGridScale - 100) / (100 / mlngGridWidth))) > 0, (mlngGridWidth - ((mintGridScale - 100) / (100 / mlngGridWidth))), 1)
    mlngSqrHeight = mpicCanvas.ScaleY(mpicCanvas.Height, vbTwips, vbPixels) / _
        IIf((mlngGridHeight - ((mintGridScale - 100) / (100 / mlngGridHeight))) > 0, (mlngGridHeight - ((mintGridScale - 100) / (100 / mlngGridHeight))), 1)

    ' Indicate that the grid has been initialized yet.
    mblnInitialized = True
    
    ' Draw the initial (empty) grid.
    DrawGrid
End Sub


'
' Private Subs
'


'
' Ant Management Procedures
'


'
' Procedure AddAnt
'
' Purpose:
'   Adds a new ant to the ant collection.
'
' Parameters:
'   Key : The ant's key in the collection (must be unique).
'   X, Y (Optional) : The initial position of the ant inside the grid.
'
Public Sub AddAnt(Key As String, Optional X As Long = 0, Optional Y As Long = 0)
Dim NewAnt As clsAnt
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    If (Key <> "") Then
        ' Set the properties of the new ant.
        Set NewAnt = New clsAnt
        If ((X >= 0) And (X < mlngGridWidth) And (Y >= 0) And (Y < mlngGridHeight)) Then
            NewAnt.Initialize Key, X, Y
        Else
            NewAnt.Initialize Key, 0, 0
        End If
        
        ' Add the new ant to the collection.
        mcolAnts.Add NewAnt, Key
    End If

    ' Redraw the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Sub


'
' Procedure RemoveAnt
'
' Purpose:
'   Removes an ant from the ant collection by its key.
'
Public Sub RemoveAnt(Key As String)
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    If (Key <> "") Then _
        mcolAnts.Remove Key

    ' Redraw the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Sub


'
' Painting Procedures
'


'
' Procedure DrawGrid
'
' Purpose:
'   Draws the ant grid.
'
Private Sub DrawGrid()
' Counter variables.
Dim X As Integer, Y As Integer
' Used for drawing the grid squares.
Dim rctSqr As RECT
Dim lngBottom As Long, lngRight As Long
Dim intMaxX As Integer, intMaxY As Integer
Dim clsCurrAnt As clsAnt
Dim ptTemp As POINTAPI

    ' Don't draw the grid if the grid is disabled.
    If (Not mblnEnabled) Then Exit Sub

    ' Prepare the canvas for drawing.
    mpicCanvas.Cls
    
    If (mmodCurrentDemoMode = FastForward) Then
        ' Display a message saying the demo is in a fastForward mode.
        mpicCanvas.BackColor = vbWhite
        mpicCanvas.ForeColor = vbBlack
        mpicCanvas.CurrentX = (mpicCanvas.Width / 2) - (mpicCanvas.TextWidth(FASTForward_MSG) / 2)
        mpicCanvas.CurrentY = (mpicCanvas.Height / 2) - (mpicCanvas.TextHeight(FASTForward_MSG) / 2)
        mpicCanvas.Print FASTForward_MSG
        
        Exit Sub
    End If
    
    intMaxX = mlngGridWidth - (mintGridScale - 100) / (100 / mlngGridWidth)
    intMaxY = mlngGridHeight - (mintGridScale - 100) / (100 / mlngGridHeight)
    
    ' Draw the grid squares.
    For X = 0 To intMaxX
        For Y = 0 To intMaxY
            rctSqr.Top = Y * mlngSqrHeight
            rctSqr.Left = X * mlngSqrWidth
            rctSqr.Bottom = rctSqr.Top + mlngSqrHeight
            rctSqr.Right = rctSqr.Left + mlngSqrWidth

            ' Draw the square.
            FillRect mpicCanvas.hdc, rctSqr, IIf(mgsqGrid(X + mhsbHorizontalSB.Value - 1, Y + mvsbVerticalSB.Value - 1) = Black, mlngBlackSqrBrush, mlngWhiteSqrBrush)
        Next
    Next
    
    If (mblnShowAntPos) Then
        ' Draw the ants' positions.
        For Each clsCurrAnt In mcolAnts
            ' See if the ant is within the limits of the user's view port.
            If (((intMaxX + mhsbHorizontalSB.Value - 1) - clsCurrAnt.X >= 0) And _
                ((intMaxY + mvsbVerticalSB.Value - 1) - clsCurrAnt.Y >= 0)) Then
                rctSqr.Top = (clsCurrAnt.Y - (mvsbVerticalSB.Value - 1)) * mlngSqrHeight
                rctSqr.Left = (clsCurrAnt.X - (mhsbHorizontalSB.Value - 1)) * mlngSqrWidth
                rctSqr.Bottom = rctSqr.Top + mlngSqrHeight
                rctSqr.Right = rctSqr.Left + mlngSqrWidth
                
                ' Draw the square.
                FillRect mpicCanvas.hdc, rctSqr, mlngAntBrush
            End If
        Next
    End If

    If (mblnShowGridLines) Then
        ' Draw the grid lines.
        
        mpicCanvas.ForeColor = mlngGridLinesColor
        
        lngRight = mpicCanvas.ScaleX(mpicCanvas.Width, vbTwips, vbPixels)
        lngBottom = mpicCanvas.ScaleY(mpicCanvas.Height, vbTwips, vbPixels)

        ' Draw vertical grid lines.
        For X = 0 To intMaxX
            MoveToEx mpicCanvas.hdc, X * mlngSqrWidth, 0, ptTemp
            LineTo mpicCanvas.hdc, X * mlngSqrWidth, lngBottom
        Next
        
        ' Draw horizontal grid lines.
        For Y = 0 To intMaxY
            MoveToEx mpicCanvas.hdc, 0, Y * mlngSqrHeight, ptTemp
            LineTo mpicCanvas.hdc, lngRight, Y * mlngSqrHeight
        Next
    End If

    ' Refresh the canvas.
    mpicCanvas.Refresh
End Sub


Private Sub ResizeControls()
    ' If the grid hasn't been initialized yet, exit the sub.
    If (Not mblnInitialized) Then Exit Sub
    
    ' Make the controls invisible during the resizing process.
    mpicCanvas.Visible = False
    mvsbVerticalSB.Visible = False
    mhsbHorizontalSB.Visible = False
    
    ' Resize the controls.
    
    mvsbVerticalSB.Top = 0
    mvsbVerticalSB.Left = mpicCanvas.Width - VSB_WIDTH
    mvsbVerticalSB.Width = VSB_WIDTH
    mvsbVerticalSB.Height = mpicCanvas.Height - HSB_HEIGHT
    
    mhsbHorizontalSB.Top = mpicCanvas.Height - HSB_HEIGHT
    mhsbHorizontalSB.Left = 0
    mhsbHorizontalSB.Width = mpicCanvas.Width - VSB_WIDTH
    mhsbHorizontalSB.Height = HSB_HEIGHT

    ' Show the controls back again.
    mpicCanvas.Visible = True
    mvsbVerticalSB.Visible = True
    mhsbHorizontalSB.Visible = True

End Sub


'
' Demo Control Procedures
'


'
' Procedure RunDemo
'
' Purpose:
'   Runs the Langston's Ant demo.
'
Private Sub RunDemo()
' Used for looping through all ants on the grid.
Dim clsCurrAnt As clsAnt
' Used for drawing the grid squares.
Dim rctSqr As RECT
Dim ptTemp As POINTAPI
Dim intMaxX As Integer, intMaxY As Integer

    intMaxX = mlngGridWidth - (mintGridScale - 100) / (100 / mlngGridWidth)
    intMaxY = mlngGridHeight - (mintGridScale - 100) / (100 / mlngGridHeight)

    ' Continue running while the demo is running.
    While (mmodCurrentDemoMode = Running)
        ' Move each of the ants in the grid Forward.
        For Each clsCurrAnt In mcolAnts
            If (mmodCurrentDemoMode <> Running) Then Exit For
            
            DoEvents
            
            ' Determine which way to go.
            If (mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = Black) Then
                clsCurrAnt.TurnLeft
                mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = White
            Else
                clsCurrAnt.TurnRight
                mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = Black
            End If
            
            ' See if the square to be repainted is within the limits of the user's view port.
            If (((intMaxX + mhsbHorizontalSB.Value - 1) - clsCurrAnt.X >= 0) And _
                ((intMaxY + mvsbVerticalSB.Value - 1) - clsCurrAnt.Y >= 0)) Then
                ' Repaint the grid square which was altered.
                rctSqr.Top = (clsCurrAnt.Y - (mvsbVerticalSB.Value - 1)) * mlngSqrHeight
                rctSqr.Left = (clsCurrAnt.X - (mhsbHorizontalSB.Value - 1)) * mlngSqrWidth
                rctSqr.Bottom = rctSqr.Top + mlngSqrHeight
                rctSqr.Right = rctSqr.Left + mlngSqrWidth
                
                ' Draw the square.
                FillRect mpicCanvas.hdc, rctSqr, IIf(mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = Black, mlngBlackSqrBrush, mlngWhiteSqrBrush)
            
                If (mblnShowGridLines) Then
                    ' Redraw the grid lines surrounding the square which was repainted.
                    
                    mpicCanvas.ForeColor = mlngGridLinesColor
                    
                    MoveToEx mpicCanvas.hdc, rctSqr.Left, rctSqr.Top, ptTemp
                    LineTo mpicCanvas.hdc, rctSqr.Left, rctSqr.Bottom
                    LineTo mpicCanvas.hdc, rctSqr.Right, rctSqr.Bottom
                    LineTo mpicCanvas.hdc, rctSqr.Right, rctSqr.Top
                    LineTo mpicCanvas.hdc, rctSqr.Left, rctSqr.Top
                    
                    DoEvents
                End If
            End If
            
            ' Allow the user to do other things while the demo is running.
            DoEvents
            
            ' Move the ant Forward.
            clsCurrAnt.MoveForward
            
            ' See if the ant is out of the grid borders.
            If (clsCurrAnt.X > mlngGridWidth) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, 0, clsCurrAnt.Y, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.X < 0) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, mlngGridWidth, clsCurrAnt.Y, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.Y > mlngGridHeight) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, clsCurrAnt.X, 0, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.Y < 0) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, clsCurrAnt.X, mlngGridHeight, clsCurrAnt.Direction, clsCurrAnt.Rounds
            
            ' Allow the user to do other things while the demo is running.
            DoEvents
            
            ' See if the square to be repainted is within the limits of the user's view port.
            If ((mblnShowAntPos) And ((intMaxX + mhsbHorizontalSB.Value - 1) - clsCurrAnt.X >= 0) And _
                ((intMaxY + mvsbVerticalSB.Value - 1) - clsCurrAnt.Y >= 0)) Then
                ' Since the "Show Ant Position" option is enabled, we must repaint the
                ' grid square which the ant is currently at.
                rctSqr.Top = (clsCurrAnt.Y - (mvsbVerticalSB.Value - 1)) * mlngSqrHeight
                rctSqr.Left = (clsCurrAnt.X - (mhsbHorizontalSB.Value - 1)) * mlngSqrWidth
                rctSqr.Bottom = rctSqr.Top + mlngSqrHeight
                rctSqr.Right = rctSqr.Left + mlngSqrWidth
                
                ' Draw the square.
                FillRect mpicCanvas.hdc, rctSqr, mlngAntBrush
                
                If (mblnShowGridLines) Then
                    ' Redraw the grid lines surrounding the square which was repainted.
                    
                    mpicCanvas.ForeColor = mlngGridLinesColor
                    
                    DoEvents
                    
                    MoveToEx mpicCanvas.hdc, rctSqr.Left, rctSqr.Top, ptTemp
                    LineTo mpicCanvas.hdc, rctSqr.Left, rctSqr.Bottom
                    LineTo mpicCanvas.hdc, rctSqr.Right, rctSqr.Bottom
                    LineTo mpicCanvas.hdc, rctSqr.Right, rctSqr.Top
                    LineTo mpicCanvas.hdc, rctSqr.Left, rctSqr.Top
                    
                    DoEvents
                End If
            End If
        
            ' Allow the user to do other things while the demo is running.
            DoEvents
        Next
        
        mlngTotalRounds = mlngTotalRounds + 1
        
        ' Refresh the canvas so the new drawings will be visible.
        mpicCanvas.Refresh
        
        ' Wait for the specified number of milliseconds.
        Wait (mlngRoundDelay)
        
        DoEvents
    Wend
End Sub


'
' Procedure FastForwardDemo
'
' Purpose:
'   Fast Forwards the Langston's Ant demo.
'
Private Sub FastForwardDemo()
' Used for looping through all ants on the grid.
Dim clsCurrAnt As clsAnt

    ' Continue running while the demo is in fastForward demo.
    While (mmodCurrentDemoMode = FastForward)
        ' Move each of the ants in the grid Forward.
        For Each clsCurrAnt In mcolAnts
            If (mmodCurrentDemoMode <> FastForward) Then _
                Exit For

            ' Allow the user to do other things while the demo is running.
            DoEvents
            
            ' Determine which way to go.
            If (mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = Black) Then
                clsCurrAnt.TurnLeft
                mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = White
            Else
                clsCurrAnt.TurnRight
                mgsqGrid(clsCurrAnt.X, clsCurrAnt.Y) = Black
            End If
            
            ' Move the ant Forward.
            clsCurrAnt.MoveForward
        
            DoEvents
                    
            ' See if the ant is out of the grid borders.
            If (clsCurrAnt.X > mlngGridWidth) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, 0, clsCurrAnt.Y, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.X < 0) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, mlngGridWidth, clsCurrAnt.Y, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.Y > mlngGridHeight) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, clsCurrAnt.X, 0, clsCurrAnt.Direction, clsCurrAnt.Rounds
            If (clsCurrAnt.Y < 0) Then _
                clsCurrAnt.Initialize clsCurrAnt.Name, clsCurrAnt.X, mlngGridHeight, clsCurrAnt.Direction, clsCurrAnt.Rounds
            
            DoEvents
        Next
        
        mlngTotalRounds = mlngTotalRounds + 1
        
        DoEvents
    Wend
End Sub


'
' Misc. Procedures
'


'
' Procedure Wait
'
' Purpose:
'   Waits for a specified number of milliseconds.
'
Private Sub Wait(Time As Long)
Dim lngStartTime As Long
    
    ' Save the initial time.
    lngStartTime = GetTickCount
    
    While ((GetTickCount - lngStartTime < Time) And (mmodCurrentDemoMode = Running))
        DoEvents
    Wend
End Sub


'
' Control Events
'


Private Sub mhsbHorizontalSB_Change()
Dim modCurrMode As DemoMode

    ' If the grid hasn't been initialized yet, exit the sub.
    If (Not mblnInitialized) Then Exit Sub

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Sub

Private Sub mvsbVerticalSB_Change()
Dim modCurrMode As DemoMode

    ' If the grid hasn't been initialized yet, exit the sub.
    If (Not mblnInitialized) Then Exit Sub

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Sub

Private Sub mpicCanvas_Resize()
Dim modCurrMode As DemoMode

    ' If the grid hasn't been initialized yet, exit the sub.
    If (Not mblnInitialized) Then Exit Sub

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    ' Resize the controls.
    ResizeControls
    
    ' Set the new dimensions of the grid square.
    mlngSqrWidth = mpicCanvas.ScaleX(mpicCanvas.Width, vbTwips, vbPixels) / _
        IIf((mlngGridWidth - ((mintGridScale - 100) / (100 / mlngGridWidth))) > 0, (mlngGridWidth - ((mintGridScale - 100) / (100 / mlngGridWidth))), 1)
    mlngSqrHeight = mpicCanvas.ScaleY(mpicCanvas.Height, vbTwips, vbPixels) / _
        IIf((mlngGridHeight - ((mintGridScale - 100) / (100 / mlngGridHeight))) > 0, (mlngGridHeight - ((mintGridScale - 100) / (100 / mlngGridHeight))), 1)
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
End Sub


Private Sub Class_Terminate()

    ' Terminate the class and the controls created by it.
    
    ' Used when the object to be unloaded was already unloaded (occurs when a form
    ' is unloaded before the class is terminated).
    On Error Resume Next

    mmodCurrentDemoMode = Stopped
    
    mblnInitialized = False
    
    mvntControls.Remove mvsbVerticalSB
    mvntControls.Remove mhsbHorizontalSB
End Sub


'
' File Manipulation Procedures
'


'
' Procedure SaveGrid
'
' Purpose:
'   Saves a grid to a file.
'
Public Sub SaveGrid(FileName As String)
' Used for looping through all ants on the grid.
Dim clsCurrAnt As clsAnt
' The number of the file being saved.
Dim intGridFile As Integer
' Used for setting indivisual bits.
Dim bytCurrBit As Byte, bytCurrByte As Byte
' Counter variables.
Dim X As Integer, Y As Integer
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused

    ' Open the file.
    
    intGridFile = FreeFile
    
    Open FileName For Binary As intGridFile
    
    ' First, save the header (including grid settings).
    
    Put intGridFile, , CStr("LA")
    
    Put intGridFile, , mlngAntColor
    Put intGridFile, , mlngBlackSqrColor
    Put intGridFile, , mlngWhiteSqrColor
    Put intGridFile, , mlngGridLinesColor
    
    Put intGridFile, , mlngRoundDelay
    
    Put intGridFile, , mblnShowGridLines
    Put intGridFile, , mblnShowAntPos
    
    Put intGridFile, , mlngTotalRounds
    
    ' Save the information about the ants in the grid.
    
    Put intGridFile, , CInt(mcolAnts.Count)
    
    For Each clsCurrAnt In mcolAnts
        Put intGridFile, , CInt(Len(clsCurrAnt.Name))
        Put intGridFile, , CStr(clsCurrAnt.Name)
        Put intGridFile, , clsCurrAnt.X
        Put intGridFile, , clsCurrAnt.Y
        Put intGridFile, , clsCurrAnt.Direction
        Put intGridFile, , clsCurrAnt.Rounds
    Next
    
    ' Finally, save the grid itself.
    
    Put intGridFile, , mlngGridWidth
    Put intGridFile, , mlngGridHeight
    
    ' Save each grid square in a single bit.
    
    bytCurrBit = 1
    bytCurrByte = 0

    For X = 0 To mlngGridWidth
        For Y = 0 To mlngGridHeight
            ' Set\Unset the current bit in the current byte.
            bytCurrByte = bytCurrByte Or (bytCurrBit And IIf(mgsqGrid(X, Y) = Black, bytCurrBit, 0))
            
            ' See if we've passed the end of the current byte.
            If (bytCurrBit = 128) Then
                ' Save the current byte to the file.
                Put intGridFile, , bytCurrByte
                ' Move back to the beginning of the byte.
                bytCurrBit = 1
                ' Reset the current byte.
                bytCurrByte = 0
            Else
                ' Move to the next bit in the current byte.
                bytCurrBit = bytCurrBit * 2
            End If
        Next
    Next
    
    ' Save the remaining byte (if there is one).
    If (bytCurrBit > 1) Then
        Put intGridFile, , bytCurrByte
    End If
    
    ' Close the file.
    Close intGridFile

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode

End Sub


'
' Procedure LoadGrid
'
' Purpose:
'   Loads a grid from a file.
'
Public Sub LoadGrid(FileName As String)
' Used for looping through all ants on the grid.
Dim clsCurrAnt As clsAnt
' The number of the file being read.
Dim intGridFile As Integer
' Used for reading indivisual bits.
Dim bytCurrBit As Byte, bytCurrByte As Byte
' Counter variables.
Dim X As Long, Y As Long, i As Integer
' Temp variables used for reading data from the grid file.
Dim strTemp As String, bytTemp As Byte, intTemp As Integer, intTemp2 As Integer, lngTemp As Long, dirTemp As Direction
Dim modCurrMode As DemoMode

    ' Pause the demo if it's running.
    modCurrMode = mmodCurrentDemoMode
    mmodCurrentDemoMode = Paused
    
    On Error GoTo ErrHandler

    ' Open the file.
    
    intGridFile = FreeFile
    
    Open FileName For Binary As intGridFile
    
    ' First, save the header (including grid settings).
    
    strTemp = Space$(2)
    Get intGridFile, , strTemp
    If (strTemp <> "LA") Then Exit Sub
    
    Get intGridFile, , mlngAntColor
    mlngAntBrush = CreateSolidBrush(mlngAntColor)
    Get intGridFile, , mlngBlackSqrColor
    mlngBlackSqrBrush = CreateSolidBrush(mlngBlackSqrColor)
    Get intGridFile, , mlngWhiteSqrColor
    mlngWhiteSqrBrush = CreateSolidBrush(mlngWhiteSqrColor)
    Get intGridFile, , mlngGridLinesColor
    
    Get intGridFile, , mlngRoundDelay
    
    If (mlngRoundDelay < 0) Then Exit Sub
    
    Get intGridFile, , mblnShowGridLines
    Get intGridFile, , mblnShowAntPos
    
    Get intGridFile, , mlngTotalRounds
    
    ' Save the information about the ants in the grid.
    
    Get intGridFile, , intTemp
    
    If (intTemp > 0) Then Set mcolAnts = New Collection
    
    For i = 1 To intTemp
    
        ' Initialize the current ant.
        Set clsCurrAnt = New clsAnt
        
        Get intGridFile, , intTemp2 '
        strTemp = Space$(intTemp2)  ' Read the string buffer.
        Get intGridFile, , strTemp  '
        Get intGridFile, , X
        Get intGridFile, , Y
        Get intGridFile, , dirTemp
        Get intGridFile, , lngTemp
        
        clsCurrAnt.Initialize strTemp, X, Y, dirTemp, lngTemp
        
        ' Add the ant to the ant collection.
        mcolAnts.Add clsCurrAnt, strTemp
    Next
    
    ' Finally, save the grid itself.
    
    Get intGridFile, , mlngGridWidth
    Get intGridFile, , mlngGridHeight
    
    If ((mlngGridWidth <= 0) Or (mlngGridHeight <= 0)) Then Exit Sub
    
    ' Save each grid square in a single bit.
    
    ReDim mgsqGrid(0, 0)
    ReDim mgsqGrid(mlngGridWidth, mlngGridHeight)
    
    bytCurrBit = 1
    Get intGridFile, , bytCurrByte
    
    For X = 0 To mlngGridWidth
        For Y = 0 To mlngGridHeight
            ' Set\Unset the current square by reading the current bit.
            mgsqGrid(X, Y) = IIf(bytCurrByte And bytCurrBit, Black, White)
            
            ' See if we've passed the end of the current byte.
            If (bytCurrBit = 128) Then
                ' Load the next byte from the file.
                Get intGridFile, , bytCurrByte
                ' Move back to the beginning of the byte.
                bytCurrBit = 1
            Else
                ' Move to the next bit in the current byte.
                bytCurrBit = bytCurrBit * 2
            End If
        Next
    Next
    
ErrHandler:
    
    ' Close the file.
    Close intGridFile
    
    ' Repaint the grid.
    DrawGrid

    ' Rerun the demo if it was paused.
    mmodCurrentDemoMode = modCurrMode
    
End Sub


'
' Procedure SaveGridPicture
'
' Purpose:
'   Saves a snapshot of the grid in a bitmap file.
'
Public Sub SaveGridPicture(FileName As String)
    ' Save a snapshot of the grid into a bitmap file.
    Call SavePicture(mpicCanvas.Image, FileName)
End Sub
